# -*- coding: utf-8 -*-

"""Tools for Xtriage"""

# ----------------------- Imports --------------------------------
import os
import subprocess
from pathlib import Path
# import chardet # TODO à voir: pour dédecter un encodage, nécessite son installation avec pip
# http://projet-lvmmw.overblog.com/python-verification-de-l-encodage-et-extraction-du-contenu-textuel
# My modules
from modules.utils import proteins
from modules.utils import software
import config


#--------------------------Functions------------------------------
# Setup Phenix
if os.name == 'posix':
    env_phenix = software.shell_source(Path(config.PHENIX_PATH).joinpath("phenix_env.sh"))  # sourcer Phenix sous Linux
    shell_choice = False
elif os.name == 'nt':
    env_phenix = software.setup_bat(Path(config.PHENIX_PATH).joinpath("phenix_env.bat"))  # sourcer Phenix sous Windows
    shell_choice = True

def xtriage_mtz(mtz, xtriage_verbose, xtriage_log):
    """
    :param mtz: mtz file with path
    :param xtriage_verbose: file name with path
    :param xtriage_log: file name with path
    Run Xtriage on an mtz file.
    Manages error 'Multiple equally suitable arrays of observed xray data found.',
    by running again with 'scaling.input.xray_data.obs_labels', ex: "scaling.input.xray_data.obs_labels=IMEAN,SIGIMEAN"
    Write out and error in file xtriage_verb, write log in file xtriage_log.
    """
    # ne marche pas en windows sans 'shell=True'
    # TODO: ajouter text=True à la place de encoding='utf-8' quand python >= 3.7, et xtriage_verb.write(out), xtriage_verb.write(err)
    xtriage = subprocess.Popen(["phenix.xtriage", str(mtz), "parameters.reporting.log=" + str(xtriage_log)],
                               stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=shell_choice, env=env_phenix, encoding='utf-8')

    with open(xtriage_verbose, "w") as xtriage_verb:
        out, err = xtriage.communicate()
        xtriage_verb.write(out)
        xtriage_verb.write(err)
    if not xtriage_log.exists():  # erreur pas de fichier xtriage créé
        labels = ""  # labels à indiquer à Xtriage
        with open(xtriage_verbose, "r") as xtriage_verb:
            lignes = xtriage_verb.readlines()
            for i, l in enumerate(lignes[::-1]):  # on parcourt à l'envers
                if "Possible choices:" in l:
                    # ligne juste après
                    labels = lignes[len(lignes) - i].rsplit(':',1)[1]   # split on last delimiter (cas où D: par ex dans le chemin du fichier)
        # TODO: ajouter text=True à la place de encoding='utf-8' quand python >= 3.7, et xtriage_verb.write(out), xtriage_verb.write(err)
        xtriage = subprocess.Popen(["phenix.xtriage", str(mtz), "parameters.reporting.log=" + str(xtriage_log),
                                    "scaling.input.xray_data.obs_labels=" + labels],
                                   stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=shell_choice, env=env_phenix, encoding='utf-8')
        xtriage_verbose = mtz.parent.joinpath("xtriage-verbose_labels.log")
        with open(xtriage_verbose, "w") as xtriage_verb:
            out, err = xtriage.communicate()
            xtriage_verb.write(out)
            xtriage_verb.write(err)


def reso_in_xtriage_log(logfile):
    """
        Get space groupe in log file from Xtriage.
        :param logfile: the log file from Xtriage
        :return: the value for resolution
        """
    reso = "not found"
    with open(logfile, 'r') as lf:
        lignes = lf.readlines()
        for l in lignes:
            if l.find("Resolution:") >= 0:
                reso = round(float(l.split('Resolution:')[1].split('-')[1].strip()), 2)
    return reso


def completeness_in_xtriage_log(logfile):
    """
        Get completeness in log file from Xtriage.
        :param logfile: the log file from Xtriage
        :return: the value for completeness
        """
    compl = "not found"
    with open(logfile, 'r') as lf:
        lignes = lf.readlines()
        for l in lignes:
            if l.find("Completeness:") >= 0:
                compl = float(l.split("Completeness:")[1].split("%")[0])
                break
            if l.find("Completeness (all):") >= 0:
                 compl = float(l.split("Completeness (all):")[1].split("%")[0])
    return compl

def sg_in_xtriage_log(logfile):
    """
        Get space group in log file from Xtriage.
        :param logfile: the log file from Xtriage
        :return: the value for space group
        """
    sg = "not found"
    with open(logfile, 'r') as lf:
        lignes = lf.readlines()
        for l in lignes:
            if l.find("Space group:") >= 0:
                if "(No." in l.split("Space group:")[1]:        # chercher avec le No car permet de gérer P 1 2 1 = P2
                    sg = proteins.sg_nb_to_symbol[int(l.split("Space group:")[1].split('(No.')[1].split(')')[0].strip())]
                else:
                    sg = ''.join(l.split("Space group:")[1].split()).strip()
                    if sg not in proteins.sg_nb_to_symbol.values():
                        sg = "not found"
                break
    return sg

def unitcell_in_xtriage_log(logfile):
    """
        Get space group in log file from Xtriage.
        :param logfile: the log file from Xtriage
        :return: the value for unit cell
        """
    uc = "not found"
    with open(logfile, 'r') as lf:
        lignes = lf.readlines()
        for l in lignes:
            if l.find("Unit cell:") >= 0:
                uc = l.split("Unit cell:")[1].strip().strip('()') # cas possible paramètres entre parenthèses
    return uc
